import { BaseCommand } from '../../base.js';
import { hasProfile, removeProfile } from '../../credentials.js';
class Logout extends BaseCommand {
    async run() {
        const { flags } = await this.parseCommand();
        const profile = await this.getProfile({ ignoreEnv: true });
        const loggedIn = await hasProfile(profile.name);
        if (!loggedIn) {
            return this.error('You are not logged in');
        }
        const { confirm } = await this.prompt({
            type: 'confirm',
            name: 'confirm',
            message: 'Are you sure you want to logout of Xata?',
            initial: true
        }, flags.force);
        if (!confirm)
            this.exit(2);
        await removeProfile(profile.name);
        this.success('Logged out correctly');
    }
}
Logout.description = 'Logout from Xata';
Logout.examples = [];
Logout.flags = {
    ...BaseCommand.forceFlag()
};
Logout.args = {};
export default Logout;
