var _a;
import { Args, Flags } from '@oclif/core';
import { BaseCommand } from '../../base.js';
class DatabasesRename extends BaseCommand {
    async run() {
        const { args, flags } = await this.parseCommand();
        const workspace = flags.workspace || (await this.getWorkspace());
        const database = args.database || (await this.getDatabase(workspace, { allowCreate: false })).name;
        const newName = args.newName ||
            (await this.prompt({ type: 'text', name: 'newName', message: 'Enter the new database name' })).newName;
        const xata = await this.getXataClient();
        const { confirm } = await this.prompt({
            type: 'text',
            name: 'confirm',
            message: `Renaming the database from ${database} to ${newName} will also change your endpoint URL. This will require code changes in your application.\nPlease type: ${database} to confirm:\n`,
            initial: false
        }, flags.force ? database : undefined);
        if (!confirm)
            return this.exit(1);
        if (confirm !== database)
            return this.error('The database name did not match');
        await xata.api.databases.renameDatabase({
            pathParams: { workspaceId: workspace, dbName: database },
            body: { newName }
        });
        if (this.jsonEnabled())
            return {};
        this.success(`Database ${workspace}/${database} successfully renamed to ${workspace}/${newName}`);
    }
}
_a = DatabasesRename;
DatabasesRename.description = 'Rename a database';
DatabasesRename.examples = [];
DatabasesRename.flags = {
    ..._a.commonFlags,
    ...BaseCommand.forceFlag(),
    workspace: Flags.string({
        description: 'Workspace id the database to delete belongs to'
    })
};
DatabasesRename.args = {
    database: Args.string({ description: 'The existing database to rename', required: false }),
    newName: Args.string({ description: 'The new database name', required: false })
};
DatabasesRename.enableJsonFlag = true;
export default DatabasesRename;
